@@ -1,23 +1,3 @@
-## Solve a System of Equations
-## 
-## Binary operator that solves the equation A %*% x = b for x, where b can be 
-## either a vector or a matrix.
-## 
-## @param A A square numeric or complex matrix containing the coefficients of 
-##          the linear system. Logical matrices are coerced to numeric.
-## @param b A numeric or complex vector or matrix giving the right-hand side(s) 
-##          of the linear system. If missing, \code{b} is taken to be an 
-##          identity matrix and solve will return the inverse of \code{A}.
-## @export
-## @examples
-## A <- matrix(rnorm(4), 2, 2)
-## b <- c(1, 2)
-## A %\% b  
-##`%\\%` <- function(A, b) {
-##  solve(A, b)
-##}
-
-
 #' Row/Column Max/Min Indices
 #'
 #' Returns the indices of the maximum or minimum values along an axis.
@@ -26,9 +6,7 @@
 #' @param rows If \code{TRUE} (the default) the indices of each row max/min is
 #'             returned. 
 #' @return A vector of indices.
-#' 
 #' @export
-#' 
 #' @examples
 #' m <- mat("94, 20, 44; 40, 92, 51; 27, 69, 74")
 #' argmax(m)
@@ -58,21 +36,17 @@ argmin <- function(x, rows = TRUE) {
 #' 
 #' @param nrow The desired number of rows.
 #' @param ncol The desired number of columns.
-#' 
-#' @return A \code{nrow}-by-\code{ncol} identity matrix.
-#' 
+#' @return A \code{nrow}-by-\code{ncol} identity matrix of class 
+#'         \code{c("matrix", "mat")}.
 #' @seealso \code{\link{diag}}.
 #' @export
-#' 
 #' @examples
 #' eye(4)  # 4-by-4 identity matrix
 #' eye(4, 4)  # 4-by-4 identity matrix
 #' eye(3, 5)  # 3-by-5 identity matrix
 #' eye(5, 3)  # 5-by-3 identity matrix
 eye <- function(nrow = 1, ncol = nrow) {
-  m <- diag(1L, nrow, ncol)
-  class(m) <- c("matrix", "mat")
-  m
+  diag(1L, nrow, ncol)
 }
 
 
@@ -84,22 +58,29 @@ eye <- function(nrow = 1, ncol = nrow) {
 #' @param nrow The desired number of rows.
 #' @param ncol The desired number of columns.
 #' @param ... Further dimensions of the array.
-#' 
-#' @return A matrix or array filled with the value \code{x}.
-#' 
-#' @export
+#' @param atleast_2d Logical indicating whether or not to force column vectors 
+#'   to have a second dimension equal to one. Defaults to \code{FALSE}. This 
+#'   behavior can also be changed globally using, for example 
+#'   \code{options(atleast_2d = TRUE)}.
+#' @return A matrix or array filled with the value \code{x}. If the result is a
+#'         matrix, it will have class \code{c("matrix", "mat")}.
 #' @seealso \code{\link{ones}}, \code{\link{zeros}}, \code{\link{falses}}, 
 #'   \code{\link{trues}}, \code{\link{mat}}, \code{\link{matrix}}.
-#' 
+#' @export
 #' @examples
 #' fill(pi, 3, 5)  # 3-by-5 matrix filled with the value of pi
 #' fill(pi, 3, 5, 2, 2)  # 3-by-5-by-2-by-2 array filled with the value of pi
 #' pi * ones(3, 5)
-fill <- function(x, nrow = 1, ncol = 1, ...) {
+#' zeros(10)
+#' zeros(10, atleast_2d = TRUE)
+fill <- function(x, nrow = 1, ncol = 1, ..., atleast_2d = NULL) {
   if (length(list(...)) == 0) {
-    m <- matrix(x, nrow = nrow, ncol = ncol)
-    class(m) <- c("matrix", "mat")
-    m
+    if (is.null(atleast_2d)) atleast_2d <- getOption("atleast_2d")
+    if (!atleast_2d && ncol == 1) {  # no dim attribute!
+      rep_len(x, length.out = nrow)
+    } else {
+      matrix(x, nrow = nrow, ncol = ncol)
+    }
   } else {
     array(x, dim = c(nrow, ncol, unlist(list(...))))
   }
@@ -108,26 +89,26 @@ fill <- function(x, nrow = 1, ncol = 1, ...) {
 
 #' @rdname fill
 #' @export
-falses <- function(nrow = 1, ncol = 1, ...) {
-  fill(FALSE, nrow = nrow, ncol = ncol, ...)
+falses <- function(nrow = 1, ncol = 1, ..., atleast_2d = NULL) {
+  fill(FALSE, nrow = nrow, ncol = ncol, ..., atleast_2d = atleast_2d)
 }
 
 #' @rdname fill
 #' @export
-trues <- function(nrow = 1, ncol = 1, ...) {
-  fill(TRUE, nrow = nrow, ncol = ncol, ...)
+trues <- function(nrow = 1, ncol = 1, ..., atleast_2d = NULL) {
+  fill(TRUE, nrow = nrow, ncol = ncol, ..., atleast_2d = atleast_2d)
 }
 
 #' @rdname fill
 #' @export
-ones <- function(nrow = 1, ncol = 1, ...) {
-  fill(1L, nrow = nrow, ncol = ncol, ...)
+ones <- function(nrow = 1, ncol = 1, ..., atleast_2d = NULL) {
+  fill(1L, nrow = nrow, ncol = ncol, ..., atleast_2d = atleast_2d)
 }
 
 #' @rdname fill
 #' @export
-zeros <- function(nrow = 1, ncol = 1, ...) {
-  fill(0L, nrow = nrow, ncol = ncol, ...)
+zeros <- function(nrow = 1, ncol = 1, ..., atleast_2d = NULL) {
+  fill(0L, nrow = nrow, ncol = ncol, ..., atleast_2d = atleast_2d)
 }
 
 
@@ -139,19 +120,15 @@ zeros <- function(nrow = 1, ncol = 1, ...) {
 #' @param across Character string specifying whether to flatten the matrix 
 #'   across \code{"rows"} (default) or \code{"columns"}. This option is ignored
 #'   for multi-way arrays.
-#' 
 #' @return A numeric vector.
-#' 
-#' @export
 #' @seealso \code{\link{mat}}.
-#' 
+#' @export
 #' @examples
 #' m <- mat("2, 4, 6, 8; 10, 12, 14, 16")
 #' flatten(m)
 #' flatten(m, across = "columns")
 flatten <- function(x, across = c("rows", "columns")) {
   if (is.matrix(x)) {
-    if (is.mat(x)) class(x) <- "matrix"  # remove "mat" class
     across <- match.arg(across)
     if (across == "rows") x <- t(x)
   } 
@@ -166,10 +143,10 @@ flatten <- function(x, across = c("rows", "columns")) {
 #' 
 #' @param x A square numeric or complex matrix
 #' @param ... Additional optional arguments.
-#' 
-#' @export
 #' @seealso \code{\link{solve}}.
-#' 
+#' @details See the documentation for the \code{base} function 
+#'         \code{\link{solve}}.
+#' @export
 #' @examples
 #' m <- 3 * eye(5)
 #' inv(m)
@@ -182,9 +159,8 @@ inv <- function(x, ...) {
   }
   b <- diag(1, nrow(x))
   colnames(b) <- rownames(x)
-  m <- solve(x, b, ...)
-  class(m) <- c("matrix", "mat")
-  m
+  solve(x, b, ...)
+
 }
 
 
@@ -193,31 +169,28 @@ inv <- function(x, ...) {
 #' Concatenate matrices along the first or second dimension.
 #' 
 #' @param ... Vectors or matrices.
-#' 
-#' @export
+#' @return A matrix of class \code{c("matrix", "mat")} formed by combining the
+#'          \code{...} arguments column-wise (\code{hcat}) or row-wise 
+#'          (\code{vcat}).
 #' @seealso \code{\link{bmat}}, \code{\link{cbind}}, \code{\link{rbind}}.
-#' 
+#' @export
 #' @examples
 #' m1 <- mat("1, 2, 3; 4, 5, 6")
 #' m2 <- mat("7, 8, 9; 10, 11, 12")
 #' hcat(m1, m2)  # same as 'bmat("m1, m2")'
 #' vcat(m1, m2)  # same as 'bmat("m1; m2")'
 hcat <- function(...) {
-  m <- do.call(cbind, list(...))
-  class(m) <- c("matrix", "mat")
-  m
+  do.call(cbind, list(...))
 }
 
 #' @rdname hcat
 #' @export
 vcat <- function(...) {
-  m <- do.call(rbind, list(...))
-  class(m) <- c("matrix", "mat")
-  m
+  do.call(rbind, list(...))
 }
 
 
-#' linearly-spaced Elements
+#' Linearly-spaced Elements
 #' 
 #' Construct a vector of \code{n} linearly-spaced elements from \code{a} 
 #' to \code{b}. 
@@ -225,12 +198,9 @@ vcat <- function(...) {
 #' @param a The starting value of the sequence.
 #' @param b The final value of the sequence.
 #' @param n The number of samples to generate. Default is 50.
-#' 
 #' @return A vector of linearly-spaced elements.
-#' 
-#' @export
 #' @seealso \code{\link{logspace}}, \code{\link{seq}}.
-#' 
+#' @export
 #' @examples
 #' linspace(0, 1)
 #' linspace(1, 5, 5)
@@ -250,16 +220,12 @@ linspace <- function(a, b, n = 50) {
 #' @param b \code{base^b} is the final value of the sequence.
 #' @param n The number of samples to generate. Default is 50.
 #' @param base The base of the log space.
-#' 
+#' @return A vector of logarithmically-spaced elements.
 #' @note
 #' If \code{b = pi} and \code{base = 10}, the points are between 
 #' \code{10^a} and \code{pi}, not \code{10^a} and \code{10^pi}, for 
 #' compatibility with the corresponding MATLAB/Octave, and NumPy functions.
-#' 
-#' @return A vector of logarithmically-spaced elements.
-#' 
 #' @seealso \code{\link{linspace}}, \code{\link{seq}}.
-#' 
 #' @export
 logspace <- function(a, b, n = 50, base = 10) {
   if (b == pi && base == 10)  {
@@ -269,6 +235,31 @@ logspace <- function(a, b, n = 50, base = 10) {
 }
 
 
+#' Rectangular 2-D Grid
+#' 
+#' Creates matrices for vectorized evaluations of 2-D scalar/vector fields over 
+#' 2-D grids.
+#' 
+#' @param x Numeric vector representing the first coordinate of the grid.
+#' @param y Numeric vector representing the second coordinate of the grid.
+#' @return a list of matrices, each having class \code{c("matrix", "mat")}.
+#' @seealso \code{\link{expand.grid}}, \code{\link{outer}}.
+#' @export
+#' @examples
+#' mg <- meshgrid(linspace(-4*pi, 4*pi, 27))  # list of input matrices
+#' z <- cos(mg[[1]]^2 + mg[[2]]^2) * exp(-sqrt(mg[[1]]^2 + mg[[2]]^2)/6)
+#' image(z, axes = FALSE)  # color image
+#' contour(z, add = TRUE, drawlabels = FALSE)  # add contour lines
+meshgrid <- function(x, y = x) {
+  lenx <- length(x)
+  leny <- length(y)
+  list(matrix(rep(x, each = leny), nrow = leny, ncol = lenx),
+       matrix(rep(y, times = lenx), nrow = leny, ncol = lenx))
+  #   list(as.mat(matrix(rep(x, each = leny), nrow = leny, ncol = lenx)),
+  #        as.mat(matrix(rep(y, times = lenx), nrow = leny, ncol = lenx)))
+}
+
+
 #' Matrix/Array of Uniform Random Numbers
 #' 
 #' Construct a matrix or multi-way array of uniform random deviates.
@@ -280,21 +271,26 @@ logspace <- function(a, b, n = 50, base = 10) {
 #'   (\code{rand} only).
 #' @param max Upper limit for the uniform distribution. Must be finite. 
 #'   (\code{rand} only).
-#'   
-#' @return A  matrix or array of pseudorandom numbers.
-#' 
-#' @export
+#' @param atleast_2d Logical indicating whether or not to force column vectors 
+#'   to have a second dimension equal to one. Defaults to \code{FALSE}. This 
+#'   behavior can also be changed globally using, for example 
+#'   \code{options(atleast_2d = TRUE)}.
+#' @return A  matrix or array of pseudorandom numbers. If the result is a
+#'         matrix, it will have class \code{c("matrix", "mat")}.
 #' @seealso \code{\link{randi}}, \code{\link{randn}}, \code{\link{runif}}.
-#' 
+#' @export
 #' @examples
 #' rand(100, 100)  # 100 by 100 matrix of uniform random numbers
 #' rand(2, 3, min = 100, max = 200)  
-rand <- function(nrow = 1, ncol = 1, ..., min = 0, max = 1) {
+rand <- function(nrow = 1, ncol = 1, ..., min = 0, max = 1, atleast_2d = NULL) {
   if (length(list(...)) == 0) {
-    m <- matrix(runif(nrow * ncol, min = min, max = max), nrow = nrow, 
-                ncol = ncol)
-    class(m) <- c("matrix", "mat")
-    m
+    if (is.null(atleast_2d)) atleast_2d <- getOption("atleast_2d")
+    if (!atleast_2d && ncol == 1) {  # no dim attribute!
+      runif(nrow, min = min, max = max)
+    } else {
+      matrix(runif(nrow * ncol, min = min, max = max), nrow = nrow, 
+             ncol = ncol)
+    }
   } else {
     array(runif(nrow * ncol * prod(unlist(list(...))), min = min, max = max), 
           dim = c(nrow, ncol, unlist(list(...))))
@@ -303,7 +299,6 @@ rand <- function(nrow = 1, ncol = 1, ..., min = 0, max = 1) {
 }
 
 
-
 #' Matrix/Array of Uniform Random Integers
 #' 
 #' Construct a matrix or multi-way array of uniform random integers.
@@ -312,24 +307,29 @@ rand <- function(nrow = 1, ncol = 1, ..., min = 0, max = 1) {
 #' @param nrow The desired number of rows.
 #' @param ncol The desired number of columns.
 #' @param ... Further dimensions of the array.
-#' 
-#' @return A  matrix or array of pseudorandom numbers.
-#' 
-#' @export
+#' @param atleast_2d Logical indicating whether or not to force column vectors 
+#'   to have a second dimension equal to one. Defaults to \code{FALSE}. This 
+#'   behavior can also be changed globally using, for example 
+#'   \code{options(atleast_2d = TRUE)}.
+#' @return A matrix or array of pseudorandom numbers. If the result is a matrix, 
+#'         it will have class \code{c("matrix", "mat")}.
 #' @seealso \code{\link{rand}}, \code{\link{randn}}, \code{\link{sample}}.
-#' 
+#' @export
 #' @examples
 #' randi(2, 5, 5)
-randi <- function(imax, nrow, ncol = 1, ...) {
+randi <- function(imax, nrow, ncol = 1, ..., atleast_2d = NULL) {
   if (!is.integer(imax)) imax <- as.integer(imax)
   if (imax < 1)  {  # make sure imax is a positive integer
     stop("imax must be a positive integer.") 
   }
   if (length(list(...)) == 0) {
-    m <- matrix(sample(imax, size = nrow * ncol, replace = TRUE), nrow = nrow, 
-                ncol = ncol)
-    class(m) <- c("matrix", "mat")
-    m
+    if (is.null(atleast_2d)) atleast_2d <- getOption("atleast_2d")
+    if (!atleast_2d && ncol == 1) {  # no dim attribute!
+      sample(imax, size = nrow, replace = TRUE)
+    } else {
+      matrix(sample(imax, size = nrow * ncol, replace = TRUE), nrow = nrow, 
+             ncol = ncol)
+    }
   } else {
     array(sample(imax, size = nrow * ncol * prod(unlist(list(...))), 
                  replace = TRUE), dim = c(nrow, ncol, unlist(list(...))))
@@ -347,21 +347,27 @@ randi <- function(imax, nrow, ncol = 1, ...) {
 #' @param mean Mean for the normal distribution. (\code{randn} only).
 #' @param sd Standard deviation for the normal distribution. 
 #'   (\code{randn} only).
-#' 
-#' @return A  matrix or array of pseudorandom numbers.
-#' 
-#' @export
+#' @param atleast_2d Logical indicating whether or not to force column vectors 
+#'   to have a second dimension equal to one. Defaults to \code{FALSE}. This 
+#'   behavior can also be changed globally using, for example 
+#'   \code{options(atleast_2d = TRUE)}.
+#' @return A  matrix or array of pseudorandom numbers. If the result is a 
+#'         matrix, it will have class \code{c("matrix", "mat")}.
 #' @seealso \code{\link{rand}}, \code{\link{randi}}, \code{\link{rnorm}}.
-#' 
+#' @export
 #' @examples
 #' randn(100, 100)  # 100 by 100 matrix of standard normal random variates
 #' randn(2, 3, mean = 10, sd = 0.1)
-randn <- function(nrow = 1, ncol = 1, ..., mean = 0, sd = 1) {
+randn <- function(nrow = 1, ncol = 1, ..., mean = 0, sd = 1, 
+                  atleast_2d = NULL) {
   if (length(list(...)) == 0) {
-    m <- matrix(rnorm(nrow * ncol, mean = mean, sd = sd), nrow = nrow, 
-                ncol = ncol)
-    class(m) <- c("matrix", "mat")
-    m
+    if (is.null(atleast_2d)) atleast_2d <- getOption("atleast_2d")
+    if (!atleast_2d && ncol == 1) {  # no dim attribute!
+      rnorm(nrow, mean = mean, sd = sd)
+    } else {
+      matrix(rnorm(nrow * ncol, mean = mean, sd = sd), nrow = nrow, 
+             ncol = ncol)
+    }
   } else {
     array(rnorm(nrow * ncol * prod(unlist(list(...))), mean = mean, sd = sd), 
           dim = c(nrow, ncol, unlist(list(...))))
@@ -384,11 +390,10 @@ randn <- function(nrow = 1, ncol = 1, ..., mean = 0, sd = 1) {
 #'                       otherwise it is filled by rows. This option is ignored
 #'                       for multi-way arrays.
 #'              
-#' @return A matrix of dimension \code{nrow}-by-\code{ncol}.
-#' 
-#' @export
+#' @return A matrix of class \code{c("matrix", "mat")} with dimension 
+#'         \code{nrow}-by-\code{ncol}.
 #' @seealso \code{\link{flatten}}, \code{\link{mat}}, \code{\link{matrix}}.
-#' 
+#' @export
 #' @examples
 #' m <- 1:9
 #' resize(m)
@@ -405,10 +410,8 @@ resize <- function(x, nrow, ncol, ..., across = c("rows", "columns"),
   # Flatten and reshape/resize matrix.
   across <- match.arg(across)
   if (length(list(...)) == 0) {
-    m <- matrix(flatten(x, across = across), nrow = nrow, ncol = ncol, 
-                byrow = byrow)
-    class(m) <- c("matrix", "mat")
-    m
+    matrix(flatten(x, across = across), nrow = nrow, ncol = ncol, 
+           byrow = byrow)
   } else {
     dim(x) <- c(nrow, ncol, unlist(list(...)))
   }
@@ -422,14 +425,145 @@ resize <- function(x, nrow, ncol, ..., across = c("rows", "columns"),
 #' Retrieve the dimensions of a matrix or array.
 #' 
 #' @param x A matrix, array, or data frame.
-#' 
 #' @return The dimensions of the object.
-#' 
 #' @export
 #' @seealso \code{\link{dim}}.
-#'
+#' @examples
 #' m <- mat("1, 3, 5; 7, 9, 11")
 #' size(m)
 size <- function(x) {
   dim(x)
 }
+
+
+#' Trace of a Matrix
+#' 
+#' Sum of diagonal elements of a matrix.
+#' 
+#' @param x A matrix.
+#' @return The sum of the diagonal elements of \code{x}.
+#' @export
+#' @examples
+#' tr(ones(5, 10))
+#' x <- replicate(1000, tr(rand(25, 25)))
+#' hist(x)
+tr <- function(x) {
+  sum(diag(x))  # sum of diagonal elements
+}
+
+
+#' Lower/Upper Triangular Matrix
+#' 
+#' Construct a matrix with ones at and below the given diagonal and zeros 
+#' elsewhere.
+#' 
+#' @param nrow The desired number of rows.
+#' @param ncol The desired number of columns.
+#' @param k The sub-diagonal at and below which the matrix is filled. 
+#'          \code{k = 0} is the main diagonal, while k < 0 is below it, and 
+#'          k > 0 is above. The default is 0.
+#' @param diag Logical indicating whether to include the diagonal. Default is 
+#'             \code{TRUE}.
+#' @export
+#' @examples
+#' tri(5, 5)
+#' tri(5, 5, 2)
+#' tri(5, 5, -1)
+tri <- function(nrow, ncol = nrow, k = 0, diag = TRUE) {
+  # FIXME: Add an "upper = TRUE" option which would return t(m) instead?
+  x <- matrix(nrow = nrow, ncol = ncol)
+  if (diag) {
+    m <- as.integer(row(x) >= col(x) - k)
+  }
+  else {
+    m <- as.integer(row(x) > col(x) - k)
+  }
+  dim(m) <- dim(x)
+  m
+}
+
+
+#' Extract Lower Triangular Matrix
+#' 
+#' Extract the lower triangular part of a matrix.
+#' 
+#' @param x A matrix.
+#' @param k Diagonal above which to zero elements. \code{k = 0} (the default) is 
+#'          the main diagonal, k < 0 is below it and k > 0 is above.
+#' @param diag Logical indicating whether to include the diagonal. Default is 
+#'             \code{TRUE}.
+#' @export
+#' @examples
+#' tril(ones(5, 5))
+#' tril(ones(5, 5), diag = TRUE)
+tril <- function(x, k = 0, diag = TRUE) {
+  if (diag) {
+    m <- ifelse(row(x) >= col(x) - k, x, 0)
+  }
+  else {
+    m <- ifelse(row(x) > col(x) - k, x, 0)
+  }  
+  class(m) <- typeof(x)
+  dim(m) <- dim(x)
+  m
+}
+
+
+#' Extract Upper Triangular Matrix
+#' 
+#' Extract the upper triangular part of a matrix.
+#' 
+#' @param x A matrix.
+#' @param k Diagonal below which to zero elements. \code{k = 0} (the default) is 
+#'          the main diagonal, k < 0 is below it and k > 0 is above.
+#' @param diag Logical indicating whether to include the diagonal. Default is 
+#'             \code{TRUE}.
+#' @export
+#' @examples
+#' triu(ones(5, 5))
+#' triu(ones(5, 5), diag = FALSE)
+triu <- function(x, k = 0, diag = TRUE) {
+  if (diag) {
+    m <- ifelse(row(x) <= col(x) - k, x, 0)
+  }
+  else {
+    m <- ifelse(row(x) < col(x) - k, x, 0)
+  }  
+  class(m) <- typeof(x)
+  dim(m) <- dim(x)
+  m
+}
+
+
+#' Lower Triangular Matrix Test
+#' 
+#' Determine if a Matrix is Lower Triangular
+#' @param x A matrix
+#' @return Logical indicating whether the given matrix is lower triangular.
+#' @export
+#' @examples
+#' m <- mat("1, 0, 0, 0; -1, 1, 0, 0; -2, -2, 1, 0; -3, -3, -3, 1")
+#' is.tril(m)
+#' is.tril(eye(3, 5))
+is.tril <- function(x) {
+  # A matrix is lower triangular if all elements above the main diagonal are 
+  # zero. Any number of the elements on the main diagonal can also be zero.
+  all(x == tril(x))
+}
+
+
+#' Upper Triangular Matrix Test
+#' 
+#' Determine if a Matrix is Upper Triangular
+#' @param x A matrix
+#' @return Logical indicating whether the given matrix is lower triangular.
+#' @export
+#' @examples
+#' m <- mat("1, -1, -1, -1; 0, 1, -2, -2; 0, 0, 1, -3; 0, 0, 0, 1")
+#' is.triu(m)
+#' is.triu(eye(3, 5))
+is.triu <- function(x) {
+  # A matrix is upper triangular if all elements below the main diagonal are 
+  # zero. Any number of the elements on the main diagonal can also be zero.
+  all(x == triu(x))
+}
\ No newline at end of file
